; -----------------------------------------------------------------------------
; Quark OS Secondary Loader Code
; -----------------------------------------------------------------------------

; Setup the segment registers
	mov ax,0x0200
	mov ds,ax
	mov ax,0xB800
	mov fs,ax

; Enable The A20 Gate using 8042 Keyboard Controller Ports 60h and 64h
EnableA20:
        cli
        mov bl,0xD0
        call KeyboardWriteCommand
        call KeyboardReadData
        or al,0x02
        push ax
        mov bl,0xD1
        call KeyboardWriteCommand
        pop bx
        call KeyboardWriteData
        mov bl,0xD0
        call KeyboardWriteCommand
        call KeyboardReadData
        and al,0x02
        sti
        jnz A20Enabled		; AL is non zero if A20 Ok
	mov si,error_a20
	call PrintString
	jmp EnableA20

; Load GDT and LDT with valid descriptors
A20Enabled:
	mov ax,0x8000
        mov es,ax
        mov di,0x00
        mov si,NullGDT	; Source Address
        mov cx,0x30		; Length of GDT
        cld			; Move in forward direction
        rep movsb		; Move GDT to its new location
        lgdt [GlobalDescriptorTable]		; Load GDT register
	mov si,success_a20
	call PrintString

; Load the Kernel From Floppy
	mov ax,0x0700
	mov es,ax
	mov bx,0x0
	mov ax,0x0046		; Number of sectors = 70 ( 35 Kb Kernel)
	mov word [total_sectors],ax
	mov ax,0x4 
	mov word [start_sector],ax
	mov si,loading_str
	call PrintString
	call LoadKernelCode
	call ClearScreen

; Switch the Floppy Drive Motor Off
	mov cx,0xFFFF
    .StepperMotorOff:
	mov dx,0x3F2
	mov al,0x0
	out dx,al
        loop .StepperMotorOff
	
; Move into the Protected Mode
        cli
        mov al,11111111b
        out 0x21,al
        in al,0x70
        or al,10000000b
        out 0x70,al
        mov eax,cr0
        or al,1
        mov cr0,eax
	jmp 0x8:0x7000		; Jump to the Assembly Kernel

; Functions Used by the Secondary Loader

LoadKernelCode:
	mov ax,[total_sectors]
	cmp ax,0x0
	jz .LoadFinish
	call DosToPhysical
	call ReadFDDSector
	mov ax,[total_sectors]
	dec ax
	mov word [total_sectors],ax
	mov ax,[start_sector]
	inc ax
	mov word [start_sector],ax
	jmp LoadKernelCode
    .LoadFinish:
	ret

DosToPhysical:
	push ax
	push cx
	push dx
	mov ax,[start_sector]
	mov dx,0x00
	mov cx,0x12
	div cx
	inc dx	
	mov byte [physical_sector],dl
	mov dx,0x00
	mov cx,0x02
	div cx
	mov byte [physical_head],dl
	mov ax,[start_sector]
	mov dx,0x00
	mov cx,0x24
	div cx
	mov byte [physical_track],al
	pop dx
	pop cx
	pop ax
	ret

ResetFDD:
	mov ah,0x00
        int 0x13
	jc ResetFDD
	ret

ReadFDDSector:
        call ResetFDD
	mov ah,0x02
	mov al,0x01
	mov cl,[physical_sector]
        mov ch,[physical_track]
        mov dh,[physical_head]
        mov dl,0
        int 0x13 
	jc .ReadRetry
	add bx,0x200
	mov si,dot_str
	call PrintString
	ret
    .ReadRetry
	call ResetFDD
	jmp ReadFDDSector
	
PrintString:
        cld
	mov ah,0x0E
    .PrintLoop
        lodsb
        cmp al,0x00
        jz .PrintFinish
        int 0x10
        jmp .PrintLoop
    .PrintFinish
	ret

ClearScreen:
	mov ah,0x00
	mov al,0x03
	int 0x10
	cld
	mov ax,0xB800
	mov es,ax
	mov di,0x0000
	mov cx,2000
	mov ah,[color]
	mov al,0x20
	rep stosw
	ret

KeyboardWaitCommand:
        in al,0x64
        and al,2 
        jnz KeyboardWaitCommand 
        ret                      

KeyboardWaitData:
        in al,0x64
        and al,1
        jz KeyboardWaitData
        ret

KeyboardWriteCommand:
        call KeyboardWaitCommand
        mov al,bl
        out 0x64,al
        ret

KeyboardReadData:
        call KeyboardWaitData
        in al,0x60
        ret

KeyboardWriteData:
        call KeyboardWaitCommand
        mov al,bl
        out 0x60,al
        ret

;Global Descriptor Table
GlobalDescriptorTable:
	dw EndGDT - NullGDT - 1
	dd 0x80000

NullGDT:
	dw 0x0000
	dw 0x0000
	db 0x00
	db 0x00
	db 0x00
	db 0x00
KernelCodeGDT: 
	dw 0x0FFF
	dw 0x0000
	db 0x00
	db 0x9A
	db 0xC0
	db 0x00
KernelDataGDT:
	dw 0x0FFF
	dw 0x00
	db 0x00
	db 0x92
	db 0xC0
	db 0x00
EndGDT:

; Data used by Secondary Loader
	color db 0x0E
	error_a20  db 'A20 Init Error ! Trying again ....',0x0D,0x0A,0x00
	success_a20 db 'A20 Address Line Initialized',0x0D,0x0A,0x00
	loading_str db 'Loading the Quark OS kernel ',0x00
	dot_str db '.',0x00
	start_sector dw 0x0000
	total_sectors dw 0x0000
	physical_sector db 0x00
	physical_head db 0x00
	physical_track db 0x00

; Pad for 3 sectors
	times 1536-($-$$) db 0x00
